[前][次][番号順一覧][スレッド一覧]

ruby-changes:5198

From: knu <ko1@a...>
Date: Fri, 30 May 2008 04:19:28 +0900 (JST)
Subject: [ruby-changes:5198] Ruby:r16693 (trunk): * enum.c (enum_count, count_all_i, Init_Enumerable),

knu	2008-05-30 04:18:54 +0900 (Fri, 30 May 2008)

  New Revision: 16693

  Modified files:
    trunk/ChangeLog
    trunk/array.c
    trunk/enum.c
    trunk/test/ruby/test_array.rb
    trunk/test/ruby/test_enum.rb

  Log:
    * enum.c (enum_count, count_all_i, Init_Enumerable),
      array.c (rb_ary_count): If no argument or block is given, count
      the number of all elements.


  http://svn.ruby-lang.org/cgi-bin/viewvc.cgi/trunk/array.c?r1=16693&r2=16692&diff_format=u
  http://svn.ruby-lang.org/cgi-bin/viewvc.cgi/trunk/test/ruby/test_enum.rb?r1=16693&r2=16692&diff_format=u
  http://svn.ruby-lang.org/cgi-bin/viewvc.cgi/trunk/ChangeLog?r1=16693&r2=16692&diff_format=u
  http://svn.ruby-lang.org/cgi-bin/viewvc.cgi/trunk/enum.c?r1=16693&r2=16692&diff_format=u
  http://svn.ruby-lang.org/cgi-bin/viewvc.cgi/trunk/test/ruby/test_array.rb?r1=16693&r2=16692&diff_format=u

Index: array.c
===================================================================
--- array.c	(revision 16692)
+++ array.c	(revision 16693)
@@ -2747,13 +2747,16 @@
 
 /*
  *  call-seq:
+ *     array.count      -> int
  *     array.count(obj) -> int
  *     array.count { |item| block }  -> int
  *  
- *  Returns the number of elements which equals to <i>obj</i>.
- *  If a block is given, counts the number of elements yielding a true value.
+ *  Returns the number of elements.  If an argument is given, counts
+ *  the number of elements which equals to <i>obj</i>.  If a block is
+ *  given, counts the number of elements yielding a true value.
  *
  *     ary = [1, 2, 4, 2]
+ *     ary.count             # => 4
  *     ary.count(2)          # => 2
  *     ary.count{|x|x%2==0}  # => 3
  *
@@ -2767,7 +2770,8 @@
     if (argc == 0) {
 	VALUE *p, *pend;
 
-	RETURN_ENUMERATOR(ary, 0, 0);
+	if (!rb_block_given_p())
+	    return LONG2NUM(RARRAY_LEN(ary));
 
 	for (p = RARRAY_PTR(ary), pend = p + RARRAY_LEN(ary); p < pend; p++) {
 	    if (RTEST(rb_yield(*p))) n++;
Index: ChangeLog
===================================================================
--- ChangeLog	(revision 16692)
+++ ChangeLog	(revision 16693)
@@ -1,3 +1,9 @@
+Fri May 30 04:17:13 2008  Akinori MUSHA  <knu@i...>
+
+	* enum.c (enum_count, count_all_i, Init_Enumerable),
+	  array.c (rb_ary_count): If no argument or block is given, count
+	  the number of all elements.
+
 Fri May 30 03:12:18 2008  Akinori MUSHA  <knu@i...>
 
 	* ext/openssl/ossl_bn.c (ossl_bn_s_rand, ossl_bn_s_pseudo_rand):
Index: test/ruby/test_array.rb
===================================================================
--- test/ruby/test_array.rb	(revision 16692)
+++ test/ruby/test_array.rb	(revision 16693)
@@ -539,6 +539,7 @@
 
   def test_count
     a = @cls[1, 2, 3, 1, 2]
+    assert_equal(5, a.count)
     assert_equal(2, a.count(1))
     assert_equal(3, a.count {|x| x % 2 == 1 })
     assert_equal(2, a.count(1) {|x| x % 2 == 1 })
Index: test/ruby/test_enum.rb
===================================================================
--- test/ruby/test_enum.rb	(revision 16692)
+++ test/ruby/test_enum.rb	(revision 16693)
@@ -30,6 +30,7 @@
   end
 
   def test_count
+    assert_equal(5, @obj.count)
     assert_equal(2, @obj.count(1))
     assert_equal(3, @obj.count {|x| x % 2 == 1 })
     assert_equal(2, @obj.count(1) {|x| x % 2 == 1 })

Index: enum.c
===================================================================
--- enum.c	(revision 16692)
+++ enum.c	(revision 16693)
@@ -14,7 +14,7 @@
 #include "ruby/util.h"
 
 VALUE rb_mEnumerable;
-static ID id_each, id_eqq, id_cmp, id_next;
+static ID id_each, id_eqq, id_cmp, id_next, id_size;
 
 static VALUE
 enum_values_pack(int argc, VALUE *argv)
@@ -108,15 +108,29 @@
     return Qnil;
 }
 
+static VALUE
+count_all_i(VALUE i, VALUE memop, int argc, VALUE *argv)
+{
+    VALUE *memo = (VALUE*)memop;
+
+    memo[0]++;
+    return Qnil;
+}
+
 /*
  *  call-seq:
+ *     enum.count                   => int
  *     enum.count(item)             => int
  *     enum.count {| obj | block }  => int
  *
  *  Returns the number of items in <i>enum</i> for which equals to <i>item</i>.
  *  If a block is given, counts the number of elements yielding a true value.
  *
  *     ary = [1, 2, 4, 2]
+ *     ary.count             # => 4
  *     ary.count(2)          # => 2
  *     ary.count{|x|x%2==0}  # => 3
  *
@@ -129,8 +143,15 @@
     rb_block_call_func *func;
 
     if (argc == 0) {
-	RETURN_ENUMERATOR(obj, 0, 0);
-        func = count_iter_i;
+	if (rb_block_given_p()) {
+	    func = count_iter_i;
+	}
+	else {
+	    if (rb_respond_to(obj, id_size)) {
+		return rb_funcall(obj, id_size, 0, 0);
+	    }
+	    func = count_all_i;
+	}
     }
     else {
 	rb_scan_args(argc, argv, "1", &memo[1]);
@@ -1826,5 +1847,6 @@
     id_each = rb_intern("each");
     id_cmp  = rb_intern("<=>");
     id_next = rb_intern("next");
+    id_size = rb_intern("size");
 }
 
@@ -108,15 +108,29 @@
  *     enum.count {| obj | block }  => int
  *
- *  Returns the number of items in <i>enum</i> for which equals to <i>item</i>.
- *  If a block is given, counts the number of elements yielding a true value.
+ *  Returns the number of items in <i>enum</i>, where #size is called
+ *  if it responds to it, otherwise the items are counted through
+ *  enumeration.  If an argument is given, counts the number of items
+ *  in <i>enum</i>, for which equals to <i>item</i>.  If a block is
+ *  given, counts the number of elements yielding a true value.
  *
  *     ary = [1, 2, 4, 2]
--
ML: ruby-changes@q...
Info: http://www.atdot.net/~ko1/quickml/

[前][次][番号順一覧][スレッド一覧]